<?php

/*
 * This action hook registers our PHP class as a WooCommerce payment gateway
 */
add_filter( 'woocommerce_payment_gateways', 'primepag_gateway_class' );
function primepag_gateway_class( $gateways ) {
	$gateways[] = 'WC_Primepag_Gateway'; // your class name is here
	return $gateways;
}

/*
 * The class itself, please note that it is inside plugins_loaded action hook
 */
add_action( 'plugins_loaded', 'primepag_init_gateway_class' );
function primepag_init_gateway_class() {

	class WC_Primepag_Gateway extends WC_Payment_Gateway {

 		/**
 		 * Class constructor, more about it in Step 3
 		 */
 		public function __construct() {

					$this->id = 'primepaggateway'; // payment gateway plugin ID
					$this->icon = 'https://paggue.io/img/logo.png'; // URL of the icon that will be displayed on checkout page near your gateway name
					$this->has_fields = true; // in case you need a custom credit card form
					$this->method_title = 'Paggue.io PIX';
					$this->method_description = 'Pagamento utilizando PIX'; // will be displayed on the options page

					// gateways can support subscriptions, refunds, saved payment methods,
					// but in this tutorial we begin with simple payments
					$this->supports = array(
						'products'
					);

					// Method with all the options fields
					$this->init_form_fields();

					// Load the settings.
					$this->init_settings();
					$this->title = $this->get_option( 'title' );
					$this->description = $this->get_option( 'description' );
					$this->enabled = $this->get_option( 'enabled' );
					$this->client_id = $this->get_option( 'client_id' );
					$this->client_secret = $this->get_option( 'client_secret' );

					$this->url_de_conexao_api = $this->get_option( 'url_de_conexao_api' );
					

					// This action hook saves the settings
					add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );

					// We need custom JavaScript to obtain a token
					add_action( 'wp_enqueue_scripts', array( $this, 'primepag_payment_scripts' ) );
					
					// You can also register a webhook here
					// add_action( 'woocommerce_api_{webhook name}', array( $this, 'webhook' ) );

					// Verificar se os campos estão preenchidos
					//if (!empty($this->url_de_conexao_api) && !empty($this->client_id) && !empty($this->client_secret) &&
					//get_option( 'primepag_webhook_control' ) != date('d') || get_option( 'primepag_webhook_url' ) != $this->url_de_conexao_api.$this->client_id ) {
					//	$this->salvar_webhooks($this->url_de_conexao_api, $this->client_id, $this->client_secret);
					//}


 		}


		/*
 		   SALVAR O WEBHOOK DE NOTIFICAÇÕES
 		
		public function salvar_webhooks($url_conexao,$cid,$csecret) {

			$token = primepag_token($url_conexao,$cid,$csecret);

			if($token):

					// CANAL 1
					$curl = curl_init();

					curl_setopt_array($curl, array(
					CURLOPT_URL => $url_conexao . '/v1/webhooks/1',
					CURLOPT_RETURNTRANSFER => true,
					CURLOPT_ENCODING => '',
					CURLOPT_MAXREDIRS => 10,
					CURLOPT_TIMEOUT => 0,
					CURLOPT_FOLLOWLOCATION => true,
					CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
					CURLOPT_CUSTOMREQUEST => 'POST',
					CURLOPT_POSTFIELDS =>'{
					"url": "'.get_option("home").'/wp-json/primepag/notificacoes",
					"authorization": "authorization_value"
					}
					',
					CURLOPT_HTTPHEADER => array(
						'Content-Type: application/json',
						'Authorization: Bearer '.$token
					),
					));

					$response = curl_exec($curl);

					curl_close($curl);

					// CANAL 2
					$curl = curl_init();

					curl_setopt_array($curl, array(
					CURLOPT_URL => $url_conexao . '/v1/webhooks/2',
					CURLOPT_RETURNTRANSFER => true,
					CURLOPT_ENCODING => '',
					CURLOPT_MAXREDIRS => 10,
					CURLOPT_TIMEOUT => 0,
					CURLOPT_FOLLOWLOCATION => true,
					CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
					CURLOPT_CUSTOMREQUEST => 'POST',
					CURLOPT_POSTFIELDS =>'{
						"url": "'.get_option("home").'/wp-json/primepag/notificacoes",
						"authorization": "authorization_value"
					}
					',
					CURLOPT_HTTPHEADER => array(
						'Content-Type: application/json',
						'Authorization: Bearer '.$token
					),
					));

					$response2 = curl_exec($curl);

					curl_close($curl);

					$logger = wc_get_logger();
					
					$logger->info( "\nPAYLOAD RESPONSE CANAL 1: ", array( 'source' => 'debug-primepag-webhooks' ) );
					$logger->info( $response, array( 'source' => 'debug-primepag-webhooks' ) );

					$logger->info( "\n\nPAYLOAD RESPONSE CANAL 2: ", array( 'source' => 'debug-primepag-webhooks' ) );
					$logger->info( $response2, array( 'source' => 'debug-primepag-webhooks' ) );

					update_option( 'primepag_webhook_control', date("d") );
					update_option( 'primepag_webhook_url', $url_conexao.$cid );


			endif;
			
		}
		*/

		/**
 		 *  OPÇÕES DO PLUGIN NA TELA DE ADMINSTRAÇÃO DO WOOCOMMERCE
 		 */
 		public function init_form_fields(){

 			$this->form_fields = array(
				'enabled' => array(
					'title'       => 'Habilitar/Desabilitar',
					'label'       => 'Habilitar Paggue PIX',
					'type'        => 'checkbox',
					'description' => '',
					'default'     => 'no'
				),
//                 'enabled_pix_express' => array(
// 					'title'       => 'Habilitar/Desabilitar PIX Express',
// 					'label'       => 'Habilitar Paggue PIX Express',
// 					'type'        => 'checkbox',
// 					'description' => 'Habilitar/Desabilitar PIX Express para clientes Plataforma BETWP',
// 					'default'     => 'no'
// 				),
//                 'enabled_pix_autosaque' => array(
// 					'title'       => 'Habilitar/Desabilitar PIX Pagamento Automático (CashOut)',
// 					'label'       => 'Habilitar Paggue PIX Pagamento Automático (CashOut) para clientes Plataforma BETWP',
// 					'type'        => 'checkbox',
// 					'description' => 'Habilitar/Desabilitar PIX automático Paggue (CashOut)',
// 					'default'     => 'no'
// 				),
				'title' => array(
					'title'       => 'Título',
					'type'        => 'text',
					'description' => 'Título para ser exibido para o usuário na página de checkout.',
					'default'     => 'Paggue.io PIX',
					'desc_tip'    => true,
				),
				'description' => array(
					'title'       => 'Descrição',
					'type'        => 'textarea',
					'description' => 'Texto para ser exibido para o usuário na página de checkout.',
					'default'     => 'Pagamento usando PIX.',
				),
				

				'url_de_conexao_api' => array(
					'title'       => 'URL de conexão com a API Paggue.io',
					'type'        => 'text',
					'description'     => 'URL informada pela equipe Paggue.io',
                    'default'     => 'https://ms.paggue.io/',
					'desc_tip'    => true,
				),
				'client_id' => array(
					'title'       => 'Client ID',
					'type'        => 'text',
					'description' => 'Client ID',
					'desc_tip'    => true,
				),
				'client_secret' => array(
					'title'       => 'Client SECRET',
					'type'        => 'password',
					'description' => 'Client SECRET',
					'desc_tip'    => true,
				),

				'custom_html' => array(
					'title' => __( 'Informações sobre os webhooks', 'woocommerce' ),
					'type'  => 'title',
					'description'  => '
						<p>
							Você precisa cadastrar os WebHooks em https://portal.paggue.io/integrations para que as notificações de pagamentos funcionem.
							Insira as seguinte informações no seu painel paggue.io:
						</p>
						<p>
							Transferências: <b>'.get_option("home").'/wp-json/paggue/notificacoes</b>
						</p>
						<p>
							Pix web: <b>'.get_option("home").'/wp-json/paggue/notificacoes</b>
						</p>
						<p>
							Ambos os Webhooks, são obrigatórios!
						</p>
					',
				),

				

			);

		
	 	}

		
		public function payment_fields() {



			// PROCESSAR PARCELAS
			$total_carrinho = WC()->cart->get_cart_contents_total();
			$tot_parcelas   = 0;
			$divisor        = 1;
			$html_parcelas  = "";
			
			while($tot_parcelas<12 && $tot_parcelas<$this->max_parcelas):

				$valor_parcela = $total_carrinho / $divisor;

				if($valor_parcela>=5):
					$html_parcelas = $html_parcelas . '<option value="'.$divisor.'">'.$divisor.'x de R$'.number_format($valor_parcela,2,",",".").'</option>';
				else:
					$tot_parcelas = 13;
				endif;

				$divisor++;
				$tot_parcelas++;

			endwhile;
			// FIM PROCESSAR PARCELAS


			$html_pix = "";
			
            $html_pix = '

					                <li class="payment_method_primepag_pix">
										<input 
                                            id="primepag_payment_pix" 
                                            type="radio" 
                                            class="input-radio" 
                                            name="primepag_meio_de_pagamento_pix" 
                                            style="display:none;"
                                            value="woo-primepag-pix" checked="checked" 
                                            data-order_button_text="Pagar com PIX" >

										 <label for="primepag_payment_pix">
											Pagar com <b>PIX</b></label>

									</li>

				';
			


			


			$html = '<ul class="wc_primepag_payment_methods primepag_payment_methods primepag_methods">
						
					'.$html_pix.'

				    </ul>

				    <!-- PIX -->
					<div class="primepag-pagamento-content primepag-pagamento-pix">
				       Na próxima tela, você visualizará o código Copia e Cola e QRCODE para pagamento.
				    </div>
                    <!-- PIX -->

			';


			echo $html;

		}

		/*
		 *  CUSTOM JAVASCRIPTS PARA PAGAMENTO COM COTAPAY
		 */
	 	public function primepag_payment_scripts() {

				// APENAS EM PÁGINAS DE CARRINHO E CHECKOUT
				if ( ! is_cart() && ! is_checkout() && ! isset( $_GET['pay_for_order'] ) ) {
					return;
				}

				// PRIMEPAG HABILITADO
				if ( 'no' === $this->enabled ) {
					return;
				}

				// CASO O USUÁRIO NÃO TENHA INFORMADO O LOGIN E SENHA DE CONEXÃO
				if ( empty( $this->client_id ) && empty( $this->client_secret ) ) {

					wc_print_notice( "Erro Paggue.io: Informações de acesso à API não foram informados nas configurações do WooCommerce", "error" );

					return;

				}

				// let's suppose it is our payment processor JavaScript that allows to obtain a token
				//wp_enqueue_script( 'misha_js', 'https://www.mishapayments.com/api/token.js' );

				// and this is our custom JS in your plugin directory that works with token.js
				wp_register_script( 'woocommerce_paggue', plugins_url( '../assets/js/paggue.js?v='.date("dmYHisu"), __FILE__ ), array( 'jquery' ) );

				// in most payment processors you have to use PUBLIC KEY to obtain a token
				//wp_localize_script( 'woocommerce_misha', 'misha_params', array(
				//	'publishableKey' => $this->publishable_key
				//));

				wp_enqueue_script( 'woocommerce_paggue' );
				
				return true;
	
	 	}

		/*
 		 * Fields validation, more in Step 5
		 */
		public function validate_fields() {

		  return true;

		}

		/*
		 * We're processing the payments here, everything about it is in Step 5
		 */
		public function process_payment( $order_id ) {

		    global $woocommerce;
 
			// we need it to get any order detailes
			$order = wc_get_order( $order_id );

			//$order->payment_complete();
            $enabled     = $this->enabled;
            $enabled_pix_express = $this->enabled_pix_express;
            $enabled_pix_autosaque = $this->enabled_pix_autosaque;
            $title       = $this->title;
            $description = $this->description;
           
            $url_de_conexao_api    = $this->url_de_conexao_api;
            //$test_client_id     = $this->test_client_id;
            //$test_client_secret = $this->test_client_secret;
            $client_id          = $this->client_id;
            $client_secret      = $this->client_secret;

           
            $primepag_url           = $url_de_conexao_api;
            $primepag_client_id     = $client_id;
            $primepag_client_secret = $client_secret;
       
            

            // LOAD THE WC LOGGER
            $logger = wc_get_logger();
            $logger->info( "URL: ".$primepag_url, array( 'source' => 'debug-paggue' ) );
            $logger->info( "paggue_client_id: ".$primepag_client_id, array( 'source' => 'debug-paggue' ) );
            $logger->info( "paggue_client_secret: ".$primepag_client_secret, array( 'source' => 'debug-paggue' ) );
           
			

			$cotapay_meio_de_pagamento = $_POST['primepag_meio_de_pagamento_pix'];
			

			// PAGAR COM PIX
			if($cotapay_meio_de_pagamento=="woo-primepag-pix"):

				    $pre_token = primepag_token($primepag_url,$primepag_client_id,$primepag_client_secret);

					$ex = explode("###",$pre_token);
					$token = $ex[0];
					$companie = $ex[1];

                    $logger->info( "Paggue.io Token: ".$token, array( 'source' => 'debug-paggue' ) );
					$logger->info( "Paggue.io Companie: ".$companie, array( 'source' => 'debug-paggue' ) );

					if($pre_token):

                        $status_pagamento = primepag_pix($pre_token,$primepag_url,$order_id);

                        $logger->info( "STATUS PAGAMENTO: ".$status_pagamento, array( 'source' => 'debug-paggue' ) );
                        $logger->info( $status_pagamento, array( 'source' => 'debug-paggue' ) );

		
						if($status_pagamento["content"]):

							// LIMPAR O CARRINHO
							$woocommerce->cart->empty_cart();

							// ATUALIZAR O STATUS PARA PROCESSANDO
							//$order->update_status( 'wc-processing' );

							//wc_reduce_stock_levels( $order->get_id() );

							add_post_meta($order->get_id(),"primepag_pix","sim",true);
							add_post_meta($order->get_id(),"primepag_pix_tid",$status_pagamento["reference_code"],true);
                            add_post_meta($order->get_id(),"primepag_pix_code",$status_pagamento["content"],true);
                            add_post_meta($order->get_id(),"primepag_pix_qrcode",$status_pagamento["image_qrcode"],true);

                            // ADICIONAR NOTAS AO PEDIDO
                            $order->add_order_note( "PIX PAGGUE COPIA E COLA: ".$status_pagamento["content"] );

							// REDIRECIONAR PARA OUTRA PÁGINA
							return array(
								'result' => 'success',
								'redirect' => $this->get_return_url( $order )
							);
				 
							return true;

						else:

								error_log("NÃO CONSEGUIMOS REALIZAR PAGAMENTO COM PIX PAGGUE");
                                $logger->info( "STATUS PAGAMENTO ERRO: ".$status_pagamento, array( 'source' => 'debug-paggue' ) );

								return array(
									'result' => 'error'
								);

								return false;

						endif;	


					else:

							error_log("NÃO CONSEGUIMOS GERAR O TOKEN PAGGUE");

							return array(
								'result' => 'error'
							);

							return false;

					endif;


			endif;
			// FIM PAGAMENTO PIX


			
					
	 	}


		/*
		 *  WEBHOOK DE PROCESSAMENTO
		 */
		public function webhook() {

			// OBTER O PEDIDO
		    $order = wc_get_order( $_GET['id'] );
			
			// SETAR O PAGAMENTO COMO COMPLETO
			$order->payment_complete();

			// REDUZIR A QUANTIDADE DE ESTOQUE
			$order->reduce_order_stock();

			// DEBUG NO LOG
			update_option('webhook_debug', $_GET);

			return true;
					
	 	}
 	}
}



// RETORNO INFO DE PAGAMENTO PARA O USUÁRIO
add_action( 'woocommerce_thankyou', 'primepag_add_content_thankyou', 4 );
  
function primepag_add_content_thankyou($order_id) {

	// VERIFICAR SE O PAGAMENTO FOI FEITO
    //if (!wp_next_scheduled('executar_consultar_transacao_primepag')):
    //    wp_schedule_single_event(time(), 'executar_consultar_transacao_primepag', array(get_post_meta($order_id,"primepag_pix_tid",true), $order_id));
    //endif;

	// IMPRIMIR AS INFORMAÇÕES PARA PAGAMENTO DO BOLETO NA TELA DE OBRIGADO
	if(get_post_meta($order_id,"primepag_pix",true)=="sim"):

			echo '
                  <div class="primepag-pix-qrcode">
                  
                        <img src="'.get_post_meta($order_id,"primepag_pix_qrcode",true).'" style="padding: 12px;" />
                        
                        <h2 class="primepag-titulo-pix" style="text-align:center">
                            <small style="display:block;margin-left:auto;margin-right:auto">CÓDIGO COPIA E COLA PIX</small>
                            <span id="pixCode">'.get_post_meta($order_id,"primepag_pix_code",true).'</span>
                        </h2>
                        <p style="text-align:center">
                            <a 
                                href="javascript:void(0)" 
                                class="button button-confirmar-primepag-pix" 
                                title="COPIAR CÓDIGO"
                                style="font-size: 13px;background: #666 !important;width: fit-content;margin-bottom: 22px;">
                                COPIAR CÓDIGO
                            </a>
                        </p> 
						<!--
							<p style="text-align:center">
								<a href="javascript:void(0)" id="btnConsultaOperacaoPix" class="button button-confirmar-primepag-pix" onclick="confirmarPagamentoPIXPrimePag(`'.get_post_meta($order_id,"primepag_pix_tid",true).'`,`'.$order_id.'`)" title="JÁ FIZ O PAGAMENTO">JÁ FIZ O PAGAMENTO</a>
							</p>
						-->
						<p style="text-align:center;">
							Se você já fez o pagamento, seu pagamento será confirmado automaticamente em alguns minutos.
						</p>    

                  </div>

			';

	endif;
   
   
}



?>